# frozen_string_literal: true

module Bulkrax
  module HasLocalProcessing
    # This method is called during build_metadata
    # add any special processing here, for example to reset a metadata property
    # to add a custom property from outside of the import data
    def add_local
      if rub_importer?
        record['model'] = String.new('Dataset')
        parsed_metadata['model'] = String.new('Dataset')
        parsed_metadata['complex_person'] = complex_person
        parsed_metadata['complex_funding_reference'] = [complex_funding_reference]
        parsed_metadata['complex_date'] = map_rub_complex_date
        parsed_metadata['license'] = map_license
      else
        if record['model'] == 'Collection'
          parsed_metadata['collection_type_gid'] = collection_type.gid
        end

        return unless record['model'] == 'CrcDataset'

        map_multiple_subject
        map_multiple_modality
        map_parents
        map_size_and_format
        map_keyword
        parsed_metadata['license'] = map_crc_license
        parsed_metadata['complex_person'] = complex_person
        parsed_metadata['complex_identifier'] = complex_identifier
        parsed_metadata['complex_date'] = complex_date
        parsed_metadata['complex_funding_reference'] = [complex_funding_reference]  
        parsed_metadata['complex_relation'] = complex_relation
      end
    end

    def map_multiple_subject
      parsed_metadata['subject'] = record.fetch('subject', '').present? ? record['subject'].split(';') : []
    end

    def map_multiple_modality
      parsed_metadata['modality'] = record.fetch('modality', "").present? ? JSON.parse(record['modality']).map(&:strip) : []
    end

    def map_parents
      parsed_metadata['parents'] = record.fetch('parents', '').present? ? record['parents'].split(';').map(&:strip) : []
    end

    def map_size_and_format
      parsed_metadata['files_size'] = record.fetch('files_size', '').present? ? record['files_size'].split(';').map(&:strip) : []
      parsed_metadata['format'] = record.fetch('format', '').present? ? record['format'].split(';').map(&:strip) : []
    end

    def map_keyword
      parsed_metadata['keyword'] = record.fetch('keyword', '').present? ? record['keyword'].split(';').map(&:strip) : []
    end

    def collection_type
      Hyrax::CollectionType.find_by_title(record['collection_type'])
    end

    def complex_person
      complex_persons = []

      %w[creator contributor].each do |role|
        next unless record.fetch(role, '').present?

        full_names = record[role].split(';').map(&:strip)
        orcids = record.fetch("#{role}_orcid", '').present? ? record["#{role}_orcid"].split(';').map(&:strip) : []
        affiliations = record.fetch("#{role}_affiliation", '').present? ? record["#{role}_affiliation"].split(';').map(&:strip) : []
        # ignoring creator identifier scheme and contributor identifier scheme
        _schemes = record.fetch("#{role}_scheme", '').present? ? record["#{role}_scheme"].split(';').map(&:strip) : []
        
        full_names.each_with_index do |full_name, index|
          complex_person_object = ComplexPerson.new(RDF::Node.new, ActiveTriples::Resource.new)

          complex_person_object['last_name'], complex_person_object['first_name'] = full_name.split(',').map(&:strip)
          complex_person_object['name'] = full_name
          complex_person_object['role'] = (rub_importer? || role == 'creator') ? role : record['contributor_type']
          complex_person_object['orcid'] = orcids[index] if orcids.count == full_names.count
          complex_person_object['affiliation'] = affiliations[index] if affiliations.count == full_names.count

          complex_persons << complex_person_object
        end
      end

      complex_persons
    end

    def complex_identifier
      complex_identifiers = []

      return complex_identifiers unless record.fetch('group_identifier', '').present?
      group_ids = record['group_identifier'].split(';').map(&:strip)

      group_ids.each do |id|
        complex_identifier_object = ComplexIdentifier.new(RDF::Node.new, ActiveTriples::Resource.new)

        complex_identifier_object['scheme'] = 'group id'
        complex_identifier_object['identifier'] = id

        complex_identifiers << complex_identifier_object
      end

      complex_identifiers
    end

    def complex_date
      complex_dates = []

      # record_date
      record_dates = record.fetch('record_date','').present? ? record['record_date'].split(';').map(&:strip) : []

      record_dates.each do |record_date|
        complex_date_object = ComplexDate.new(RDF::Node.new, ActiveTriples::Resource.new)

        complex_date_object['description'] = 'Recorded'
        complex_date_object['date'] = record_date

        complex_dates << complex_date_object
      end

      # Datacite date and date type
      date_types = record.fetch('date_description', '').present? ? record['date_description'].split(';').map(&:strip) : []
      dates = record.fetch('date', '').present? ? record['date'].split(';').map(&:strip) : []
      return complex_dates unless date_types.count == dates.count

      date_types.each_with_index do |date_type, index|
        typ = DateService.new.find_by_label(date_type)
        next unless typ.present?

        complex_date_object = ComplexDate.new(RDF::Node.new, ActiveTriples::Resource.new)

        complex_date_object['description'] = typ['id']
        complex_date_object['date'] = dates[index]

        complex_dates << complex_date_object
      end

      complex_dates
    end

    def complex_subject
      complex_subject_object = ComplexSubject.new(RDF::Node.new, ActiveTriples::Resource.new)

      %w[subject_species subject_type subject_sex subject_age].each do |attribute|
        complex_subject_object[attribute] = record[attribute]
      end

      complex_subject_object
    end

    def complex_funding_reference
      complex_funding_reference_object = ComplexFundingReference.new(RDF::Node.new, ActiveTriples::Resource.new)

      %w[funder_identifier funder_name award_number award_uri award_title].each do |attribute|
        complex_funding_reference_object[attribute] = record[attribute]
      end

      complex_funding_reference_object
    end

    def complex_relation
      return [] unless record['related_identifier'].present?

      complex_relation_object = ComplexRelation.new(RDF::Node.new, ActiveTriples::Resource.new)

      complex_relation_object['title'] = 'Related resource'
      complex_relation_object['relationship'] = record['relation_type'].present? ? record['relation_type'] : 'isReferencedBy'
      complex_relation_object['url'] = record['related_identifier_type'] == 'DOI' ? "https://dx.doi.org/#{record['related_identifier']}" : record['related_identifier']

      [complex_relation_object]
    end

    def map_license
      license = LicenseService.new.find_by_label(record["license"])
      license.present? ? [license['id']] : []
    end

    def map_crc_license
      license = LicenseService.new.find_by_id(record["license"])
      license.present? ? [license['id']] : []
    end

    def map_rub_complex_date
      complex_date = []
      return complex_date unless record.fetch('date', '').present?
      record['date'].split(";").each do |date_string|
        date, date_description = date_string.split(' ', 2)
        next if date.blank? || date_description.blank?
        typ = DateService.new.find_by_label(date_description)
        next unless typ.present?

        complex_date_object = ComplexDate.new(RDF::Node.new, ActiveTriples::Resource.new)

        complex_date_object['description'] = typ['id']
        complex_date_object['date'] = date

        complex_date << complex_date_object
      end

      complex_date
    end

    def rub_importer?
      admin_set = AdminSet.find(importer.admin_set_id)
      admin_set.title.first == "RUB publication workflow"
    end
  end
end
