class User < ApplicationRecord
  # Connects this user object to Hydra behaviors.
  include Hydra::User
  # Connects this user object to Role-management behaviors.
  include Hydra::RoleManagement::UserRoles

  include RDMSRoles
  # Connects this user object to Hyrax behaviors.
  include Hyrax::User
  include Hyrax::UserUsageStats

  if Blacklight::Utils.needs_attr_accessible?
    attr_accessible :email, :password, :password_confirmation
  end
  # Connects this user object to Blacklights Bookmarks.
  include Blacklight::User
  # Include default devise modules. Others available are:
  # :registerable, :recoverable, :confirmable, :lockable, :timeoutable, :trackable
  # Did not want option to sign_up or click on forgot_password.
  devise :database_authenticatable, :rememberable, :validatable,
         :omniauthable, :omniauth_providers => [:saml, :orcid]

  # Method added by Blacklight; Blacklight uses #to_s on your
  # user class to get a user-displayable login/identifier for
  # the account.
  def to_s
    email
  end

  def can_tombstone?(presenter)
    self.admin?
  end

  def is_depositor_for?(presenter)
    presenter.depositor == self.user_key
  end

  def workflow_action_permission?(presenter)
    group_id = presenter.member_of_collection_ids.first
    state = presenter.workflow.state

    return false if state == 'draft' && !is_depositor_for?(presenter)
    return false if state == 'changes_required_from_group_manager' && !is_depositor_for?(presenter)

    admin? || is_depositor_for?(presenter) || group_manager_for?(group_id) || crc_manager_for?(group_id) || publication_manager?
  end

  def self.from_omniauth_orcid(auth)
    if ENV.fetch('ORCID_RESTRICT_AUTHORIZATION', 'true') == 'true'
      full_orcid = auth.uid
      full_orcid = "https://orcid.org/#{auth.uid}" unless auth.uid.start_with?("https://orcid.org/")
      user = User.where(orcid: full_orcid).first
    else
      user = User.find_or_create_by(orcid: auth.uid)
    end

    return nil unless user

    # Not setting user.provider and user.auth. Using them for saml auth
    user.orcid = auth.uid
    if ENV.fetch('ORCID_RESTRICT_AUTHORIZATION', 'true') == 'false'
      user.display_name = auth.info.name
      user.email = auth.info.email || "#{auth.uid}@#{auth.provider.to_s}"
      user.password = SecureRandom.random_bytes(32)
    end
    user.save!
    user
    # If you are using confirmable and the provider(s) you use validate emails,
    # uncomment the line below to skip the confirmation emails.
    # user.skip_confirmation!
  end

  def self.from_omniauth_saml(auth)
    # saml provider
    user = nil
    attributes = {
      uid: auth.extra.raw_info.attributes.fetch(ENV['SAML_ATTRIBUTE_UID'], [auth.uid]).first,
      name: auth.extra.raw_info.attributes.fetch(ENV['SAML_ATTRIBUTE_NAME'], [auth.uid]).first,
      email: auth.extra.raw_info.attributes.fetch(ENV['SAML_ATTRIBUTE_EMAIL'], ["#{auth.uid}@#{auth.provider.to_s}"]).first,
      role: auth.extra.raw_info.attributes.fetch(ENV['SAML_ATTRIBUTE_ROLE'], []),
    }
    if ENV.fetch('SAML_RESTRICT_AUTHORIZATION', 'true') == 'true'
      user = User.find_or_create_by(provider: auth.provider, uid: attributes[:uid]) if attributes[:role].map(&:downcase).include?('staff')
    else
      user = User.find_or_create_by(provider: auth.provider, uid: attributes[:uid])
    end

    return nil unless user

    user.provider = auth.provider
    user.uid = attributes[:uid]
    user.display_name = attributes[:name]
    user.email = attributes[:email]
    user.password = SecureRandom.random_bytes(32)

    user.save!
    user
    # If you are using confirmable and the provider(s) you use validate emails,
    # uncomment the line below to skip the confirmation emails.
    # user.skip_confirmation!
  end

  def self.assign_user_to_role(user, user_hash)
    user_hash = ActiveSupport::HashWithIndifferentAccess.new(user_hash)
    role_name = user_hash.fetch('role', nil)
    group_id = user_hash.fetch('group_id', nil)
    return true, '' if role_name.blank?

    role = if %w(admin publication_manager crc_1280_manager crc_1280_member).include?(role_name)
             Role.find_or_create_by(name: role_name, is_system_generated: true, is_crc_manager: role_name == 'crc_1280_manager', is_crc_member: role_name == 'crc_1280_member')
           elsif role_name == "crc_1280_group_manager" and group_id.present?
             Role.find_or_create_by(name: "crc_1280_#{group_id}_manager", is_system_generated: true, is_crc_group_manager: true)
           elsif role_name == "crc_1280_group_member" and group_id.present?
             Role.find_or_create_by(name: "crc_1280_#{group_id}_member", is_system_generated: true, is_crc_group_member: true)
           end

    unless role.present?
      if %w(crc_1280_group_manager crc_1280_group_member).include?(role_name) and group_id.blank?
        return false, "Error: #{role_name} needs a group id"
      else
        return false, "Error: #{role_name} is not a valid role"
      end
    end

    role.users << user if role.users.where(id: user.id).blank?
    role.save

    user.add_workflow_role(role)
    return true, "Assigned user #{user.email} role #{role.name}"
  end

  def self.find_by_hash(user_hash)
    user_hash = ActiveSupport::HashWithIndifferentAccess.new(user_hash)
    user = nil
    if user_hash.fetch('email', nil).present?
      user = User.find_by(email: user_hash['email'])
    elsif user_hash.fetch('saml_id', nil).present?
      user = User.find_by(provider: "saml", uid: user_hash['saml_id'])
    elsif user_hash.fetch('orcid', nil).present?
      orcid = user_hash['orcid']
      orcid = "https://orcid.org/#{user_hash['orcid']}" unless orcid.start_with?("https://orcid.org/")
      user = User.find_by(orcid: orcid)
    end
    user
  end

  def self.find_or_create_user_email(user_hash)
    user_hash = ActiveSupport::HashWithIndifferentAccess.new(user_hash)
    email = nil
    if user_hash.fetch('email', nil).present?
      email = user_hash['email']
    elsif user_hash.fetch('saml_id', nil).present?
      email = user_hash['saml_id']
      email = "#{user_hash['saml_id']}@saml" unless user_hash['saml_id'].include?('@')
    elsif user_hash.fetch('orcid', nil).present?
      email = "#{user_hash['orcid']}@orcid"
    end
    email
  end

  def self.create_user_from_hash(user_hash, update_user: false)
    user_hash = ActiveSupport::HashWithIndifferentAccess.new(user_hash)
    messages = []
    user = User.find_by_hash(user_hash)

    email = user ? user.email : User.find_or_create_user_email(user_hash)

    if user.present? and not update_user
      messages << "Warning: User #{email} already exists. Not updating user."
      return false, messages, user
    elsif user.blank? and email.blank?
      messages << "Error: Not creating user. User has no email, saml id, or orcid."
      return false, messages, nil
    elsif user.present? and update_user
      messages << "Updating user #{email}"
    elsif user.blank?
      messages << "Creating user #{email}"
      user = User.new
    end

    return false, messages, nil unless user.present?

    # email
    user.email = email

    # password
    if user_hash.fetch('password', nil).present?
      user.password = user_hash['password']
    elsif user.encrypted_password.blank?
      user.password = SecureRandom.random_bytes(32)
    end

    # display name
    user.display_name = user_hash["name"] if user_hash.fetch('name', nil).present?

    # saml_id
    if user_hash.fetch('saml_id', nil).present?
      user.provider = "saml"
      user.uid = user_hash['saml_id']
    end

    # orcid
    user.orcid = user_hash['orcid'] if user_hash.fetch('orcid', nil).present?

    # save and assign role
    if user.save
      _success, message = User.assign_user_to_role(user, user_hash)
      messages << message
    else
      messages << "Error: #{user.errors.full_messages}"
    end
    return true, messages, user
  end

  %w(add remove).each do |action|
    define_method("#{action}_workflow_role") do |role|
      case role.name
      when 'crc_1280_manager'
        handle_crc_1280_manager_role(action)
      when 'publication_manager'
        handle_publication_manager_role(action)
      when /^crc_1280_.*_manager$/
        handle_group_manager_role(action)
      when /^crc_1280_.*_member$/
        handle_group_member_role(action)
      else
        return
      end
    end
  end

  def agreement_accepted?
    agreement_accepted_at && (agreement_accepted_at > ContentBlock.for(:terms).updated_at)
  end

  private
  
  def handle_crc_1280_manager_role(action)
    admin_set = AdminSet.where(title: ENV.fetch('CRC_ADMIN_SET_TITLE', 'CRC 1280 Publishing Workflow')).first
    add_or_remove_responsibility(admin_set, 'approving_crc_manager', action)
  end
  
  def handle_publication_manager_role(action)
    titles = [
      ENV.fetch('CRC_ADMIN_SET_TITLE', 'CRC 1280 Publishing Workflow'),
      ENV.fetch('RUB_ADMIN_SET_TITLE', 'RUB publication workflow')
    ]
    admin_sets = AdminSet.where(title: titles)
    admin_sets.each do |admin_set|
      add_or_remove_responsibility(admin_set, 'approving_publication_manager', action)
    end
  end
  
  def handle_group_manager_role(action)
    admin_set = AdminSet.where(title: ENV.fetch('CRC_ADMIN_SET_TITLE', 'CRC 1280 Publishing Workflow')).first
    add_or_remove_responsibility(admin_set, 'approving_group_manager', action)
  end

  def handle_group_member_role(action)
    admin_set = AdminSet.where(title: ENV.fetch('CRC_ADMIN_SET_TITLE', 'CRC 1280 Publishing Workflow')).first
    add_or_remove_responsibility(admin_set, 'depositor', action)
  end

  def add_or_remove_responsibility(admin_set, role_name, action)
    active_workflow = admin_set.active_workflow
    workflow_role = Sipity::WorkflowRole.joins(:role).where(workflow_id: active_workflow.id, sipity_roles: {name: role_name}).first
    user_agent = Sipity::Agent.find_or_create_by(proxy_for_id: id, proxy_for_type: 'User')

    if action == 'add'
      Sipity::WorkflowResponsibility.find_or_create_by(agent_id: user_agent.id, workflow_role_id: workflow_role.id)
    else
      Sipity::WorkflowResponsibility.find_by(agent_id: user_agent.id, workflow_role_id: workflow_role.id)&.destroy
    end
  end

end
